/*
 * Setuid helper for check_solaris_nics.sh Nagios plugin.
 *
 * $Id: check_solaris_nics_nddgetdx.c,v 1.4 2008/08/27 08:09:31 kivimaki Exp $
 *
 * This setuid wrapper is part of Nagios plugin to check network
 * interfaces on Solaris. It uses ndd to get duplex information
 * of hme or e1000g drivers which cannot be acquired with kstat
 * and non-root priviledges. See "SETUID MOD" in check_solaris_nics.sh.
 *
 * Compile:
 * $ gcc -o NAGIOS_HOME/libexec/check_solaris_nics_nddgetdx \
 *    /tmp/check_solaris_nics_nddgetdx.c
 *
 * Set file permissions:
 * $ chown root:nagios NAGIOS_HOME/libexec/check_solaris_nics_nddgetdx
 * $ chmod 6550 NAGIOS_HOME/libexec/check_solaris_nics_nddgetdx
 *
 * Usage:
 * $ check_solaris_nics_nddgetdx <e1000g | hme> <instance number 0-9>
 *
 * In case of hme driver the wrapper will execute following using popen():
 *
 *   /usr/sbin/ndd -set /dev/hme instance <0-9>; /usr/sbin/ndd -get /dev/hme link_mode
 *
 * In case of e1000g driver the wrapper will execute following using popen():
 *
 *   /usr/sbin/ndd -get /dev/e1000g<0-9> link_duplex 
 *
 * Command output will be printed without any filtering or modification.
 *
 * Copyright (C) 2007-2008  Hannu Kivimki / CSC - Scientific Computing Ltd.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>

main(int argc, char *argv[]) {
    if ( argc != 3 || ( strcmp("e1000g", argv[1]) && strcmp("hme", argv[1]) ) || !isdigit(*(argv[2])) ) {
        printf("Setuid helper for check_solaris_nics.sh Nagios plugin.\n");
        printf("Copyright (C) 2007-2008  Hannu Kivimki / CSC - Scientific Computing Ltd.\n");
        printf("\n");
        printf("This setuid wrapper is part of Nagios plugin to check network\n");
        printf("interfaces on Solaris. It uses ndd to get duplex information\n");
        printf("of hme or e1000g drivers which cannot be acquired with kstat\n");
        printf("and non-root priviledges. See \"SETUID MOD\" in check_solaris_nics.sh.\n");
        printf("\n");
        printf("File permissions should be set as:\n");
        printf("  chown root:nagios %s\n", argv[0]);
        printf("  chmod 6550 %s\n", argv[0]);
        printf("\n");
        printf("Usage: %s <e1000g | hme> <instance number 0-9>\n", argv[0]);
        printf("\n");
    } else {
        char  ndd_cmd[80] = "";
        int   ndd_output_char = 0;
        FILE *ndd_output;

        setreuid(geteuid(), geteuid());
        
        if ( strcmp("e1000g", argv[1]) == 0 ) {
            sprintf(ndd_cmd, "/usr/sbin/ndd -get /dev/e1000g%c link_duplex", *(argv[2]));
        } else {
            sprintf(ndd_cmd, "/usr/sbin/ndd -set /dev/hme instance %c; /usr/sbin/ndd -get /dev/hme link_mode", *(argv[2]));
        }
        ndd_output = popen(ndd_cmd , "r");
        ndd_output_char = fgetc(ndd_output);
        while(ndd_output_char != EOF) {
            printf("%c", ndd_output_char);
            ndd_output_char = fgetc(ndd_output);
        }
        pclose(ndd_output);
    }
}
